﻿-- Включение режима отладки
local debugMode = true -- Установите true для вывода сообщений в консоль

-- Глобальные настройки поверхности для гибкости
local SurfaceSettings = {
    plowed = { friction = 0.2, resistance = 1.8, deformation = 1.0, stuckChance = 0.5, slipChance = 0.8 },
    cultivated = { friction = 0.3, resistance = 1.5, deformation = 0.8, stuckChance = 0.3, slipChance = 0.6 },
    mud = { friction = 0.1, resistance = 2.0, deformation = 1.0, stuckChance = 0.4, slipChance = 0.5 },
    grass = { friction = 0.3, resistance = 1.5, deformation = 0.6, stuckChance = 0.1, slipChance = 0.2 },
    asphalt = { friction = 0.8, resistance = 1.1, deformation = 0.2, stuckChance = 0.0, slipChance = 0.0 },
    dirt_road = { friction = 0.7, resistance = 1.4, deformation = 0.4, stuckChance = 0.2, slipChance = 0.3 },
    natural_cover = { friction = 0.6, resistance = 1.3, deformation = 0.5, stuckChance = 0.2, slipChance = 0.4 },
    default = { friction = 0.8, resistance = 1.1, deformation = 0.2, stuckChance = 0.1, slipChance = 0.2 }
}

-- Временные параметры тестирования
local testingMode = false -- Для тестов с увеличенными шансами
local testSlipChance = 0.9 -- Увеличенная вероятность пробуксовки для проверки
local testStuckChance = 0.9 -- Увеличенная вероятность застревания для проверки

-- Функция для расчёта силы тяжести и инерции
function applyGravityAndInertia(vehicle)
    local mass = vehicle:getMass() or 5000 -- Масса транспортного средства, по умолчанию 5000 кг
    local gravity = 9.8 -- Ускорение свободного падения
    local inertiaFactor = 0.05 * mass -- Инерция, пропорциональная массе

    -- Уменьшаем ускорение с учётом массы и инерции
    local currentAcceleration = vehicle:getAcceleration()
    vehicle:setAcceleration(currentAcceleration - inertiaFactor * gravity * 0.01)

    if debugMode then
        print("Gravity and inertia applied to vehicle " .. vehicle:getID() .. " (mass: " .. mass .. ")")
    end
end

-- Функция для учёта угла наклона при движении по склону
function adjustMovementOnSlope(vehicle)
    local slopeAngle = vehicle:getSlopeAngle() or 0 -- Угол наклона в градусах
    local mass = vehicle:getMass() or 5000
    local gravityForce = mass * 9.8 * math.sin(math.rad(slopeAngle)) -- Сила тяжести на наклоне

    if slopeAngle > 0 then
        -- Замедление при подъеме
        local currentSpeed = vehicle:getSpeed()
        vehicle:setSpeed(currentSpeed - gravityForce * 0.0005)
    elseif slopeAngle < 0 then
        -- Ускорение при спуске
        local currentSpeed = vehicle:getSpeed()
        vehicle:setSpeed(currentSpeed + math.abs(gravityForce * 0.0005))
    end

    if debugMode then
        print("Slope adjustment for vehicle " .. vehicle:getID() .. ": slopeAngle = " .. slopeAngle)
    end
end

-- Функция для учёта загруженности
function adjustHandlingBasedOnLoad(vehicle)
    local loadWeight = vehicle:getLoadWeight() or 0 -- Вес груза
    local baseTurningRadius = 5.0 -- Базовый радиус поворота
    local adjustedTurningRadius = baseTurningRadius + loadWeight / 1000 -- Увеличение радиуса с загрузкой

    vehicle:setTurningRadius(adjustedTurningRadius)

    if debugMode then
        print("Handling adjusted for vehicle " .. vehicle:getID() .. ": turning radius = " .. adjustedTurningRadius)
    end
end

-- Функция для изменения трения колес
function adjustWheelFrictionBasedOnWeatherAndSurface(vehicle, isRaining)
    local surfaceType = vehicle:getCurrentSurfaceType() or "default"
    local settings = SurfaceSettings[surfaceType] or SurfaceSettings["default"]
    local friction = settings.friction

    if isRaining then
        if surfaceType == "dirt_road" then
            friction = friction * 0.7 -- Уменьшение трения на грунтовой дороге в дождь (30%)
        elseif surfaceType == "natural_cover" then
            friction = friction * 0.6 -- Уменьшение трения на естественном покрытии в дождь (40%)
        else
            friction = friction * (1.0 - 0.4 * WeatherSystem:getRainIntensity()) -- Общая логика дождя
        end
    else
        if surfaceType == "dirt_road" then
            friction = friction * 0.9 -- Уменьшение трения на грунтовой дороге в сухую погоду (10%)
        elseif surfaceType == "natural_cover" then
            friction = friction * 0.8 -- Уменьшение трения на естественном покрытии в сухую погоду (20%)
        end
    end

    vehicle:setWheelFriction(friction)

    if debugMode then
        print("Friction adjusted for vehicle " .. vehicle:getID() .. " on " .. surfaceType .. ": " .. friction)
    end
end

-- Функция для полной пробуксовки вместо застревания
function simulateCompleteSlipInsteadOfStuck(vehicle)
    -- Установка параметров для 100% пробуксовки
    vehicle:setWheelSpin(3.0) -- Максимальное вращение колес для пробуксовки
    vehicle:setSpeed(0) -- Транспортное средство не движется
    vehicle:setRollingResistance(5.0) -- Увеличение сопротивления для эффекта неподвижности

    if debugMode then
        print("Vehicle " .. vehicle:getID() .. " is experiencing full slip!")
    end
end

-- Функция для проверки полного пробуксовки вместо застревания
function checkVehicleSlipInsteadOfStuck(vehicle, isRaining)
    local surfaceType = vehicle:getCurrentSurfaceType() or "default"
    local settings = SurfaceSettings[surfaceType] or SurfaceSettings["default"]
    local stuckChance = settings.stuckChance

    if isRaining then
        local rainIntensity = WeatherSystem:getRainIntensity() or 0
        stuckChance = stuckChance * (1.0 + 0.5 * rainIntensity)
    end

    -- Дополнительное увеличение вероятности для узких шин с инструментами
    if vehicle:hasNarrowTires() and vehicle:isAttachedToTool({"cultivator", "plow", "seeder"}) then
        stuckChance = stuckChance + 0.2
    end

    -- Проверка вероятности застревания, заменяем на пробуксовку
    if math.random() < stuckChance then
        simulateCompleteSlipInsteadOfStuck(vehicle)
    end
end

-- Изменение параметров почвы
function adjustTerrainDisplacement(isRaining)
    if isRaining then
        setTerrainDisplacementLayerMaxHeight(0.8)
    else
        setTerrainDisplacementLayerMaxHeight(0.4)
    end
end

function adjustFieldSoftness(isRaining)
    if isRaining then
        setTerrainResistance(0.5)
    else
        setTerrainResistance(0.8)
    end
end

-- Главная функция обновления погодных эффектов и физики техники
function updateWeatherAndPhysics()
    local isRaining = WeatherSystem:isRaining()
    for _, vehicle in ipairs(VehicleManager:getAllActiveVehicles()) do -- Только активные ТС
        adjustWheelFrictionBasedOnWeatherAndSurface(vehicle, isRaining)
        applyGravityAndInertia(vehicle)
        adjustMovementOnSlope(vehicle)
        adjustHandlingBasedOnLoad(vehicle)
        checkVehicleSlipInsteadOfStuck(vehicle, isRaining)
    end

    adjustTerrainDisplacement(isRaining)
    adjustFieldSoftness(isRaining)
end

-- Подключение к игровому циклу
EventManager:on("weatherUpdate", updateWeatherAndPhysics)